@@ -226,7 +226,7 @@ namespace giac {
     }
     gen res;
 #ifdef SMARTPTR64
-    * ((longlong * ) &res) = longlong(new ref_algext) << 16;
+    * ((ulonglong * ) &res) = ulonglong(new ref_algext) << 16;
 #else
     res.__EXTptr=new ref_algext;
 #endif
@@ -534,6 +534,7 @@ namespace giac {
 #ifndef NO_STDEXCEPT
 		}
 		catch (std::runtime_error & ){
+		  last_evaled_argptr(contextptr)=NULL;
 		  K=0;
 		}
 #endif
@@ -576,6 +577,7 @@ namespace giac {
 #ifndef NO_STDEXCEPT
 		}
 		catch (std::runtime_error & ){
+		  last_evaled_argptr(contextptr)=NULL;
 		  K=0;
 		}
 #endif
@@ -1437,6 +1439,7 @@ namespace giac {
       try {
 	tmp=limit(expr,*var._IDNTptr,*it,direction,contextptr);
       } catch (std::runtime_error & err){
+	last_evaled_argptr(contextptr)=NULL;
 	tmp=undef;
       }
 #endif
@@ -1471,6 +1474,11 @@ namespace giac {
       return gensizeerr(contextptr);
     expr=v[0];
     var=v[1];
+    // avoid inf recursion like g0(x):=ln(abs(ln(x)));
+    // g1(x,xp):=x/(ln(x))^(xp);g0(g1(x,.3));
+    gen varev=eval(var,1,contextptr); 
+    if (varev!=var && contains(varev,var))
+      return undef;
     if (expr.type==_SYMB){
       unary_function_ptr & u=expr._SYMBptr->sommet;
       if (u==at_exp || u==at_ln || u==at_atan || u==at_abs){
@@ -1661,6 +1669,7 @@ namespace giac {
 #ifndef NO_STDEXCEPT
       }
       catch (std::runtime_error & ){
+	last_evaled_argptr(contextptr)=NULL;
       }
 #endif
       unary_function_ptr s(g._SYMBptr->sommet);
@@ -1785,6 +1794,7 @@ namespace giac {
 #ifndef NO_STDEXCEPT
     }
     catch (std::runtime_error & ){
+      last_evaled_argptr(contextptr)=NULL;
       return 0;
     }
 #endif
@@ -1800,6 +1810,8 @@ namespace giac {
   }
 
   int sturmsign(const gen & g0,bool strict,GIAC_CONTEXT){
+    int fs=fastsign(g0,contextptr);
+    if (fs) return fs;
     gen g=simplifier(g0,contextptr);
     // first check some operators inv, *, exp, sqrt
     int tmp;
