@@ -49,7 +49,7 @@ namespace giac {
     bool save_cv=complex_variables(contextptr);
     complex_variables(false,contextptr);
     gen res=integrate_gen(e,x,contextptr);
-    if (lop(res,at_abs).empty()){
+    if (lop(res,at_abs).empty() && lop(res,at_floor).empty()){
       complex_variables(save_cv,contextptr);
       return res;
     }
@@ -836,12 +836,15 @@ namespace giac {
 	      bool b=calc_mode(contextptr)==1;
 	      if (b)
 		calc_mode(0,contextptr);
-	      part += _lin(_integrate(makesequence(_lin(c[i]*exp(-rac[i]*x,contextptr),contextptr),x),contextptr)*exp(rac[i]*x,contextptr),contextptr);
+	      gen tmp=_lin(c[i]*exp(-rac[i]*x,contextptr),contextptr);
+	      tmp = _integrate(makesequence(tmp,x),contextptr);
+	      part += _lin(tmp*exp(rac[i]*x,contextptr),contextptr);
 	      if (b)
 		calc_mode(1,contextptr);
 	    }
 	    if (reel && is_zero(im(cst,contextptr)))
 	      part=re(part,contextptr);
+	    //part=recursive_ratnormal(part,contextptr);
 	    part=simplify(part,contextptr);
 	  }
 	  if (step_info)
@@ -1078,6 +1081,8 @@ namespace giac {
     vx_var=save_vx;
     // Here f= f(derive(y,x),y) for a 1st order equation
     int n=diffeq_order(f,y);
+    if (n==0)
+      return solve(f,y,0,contextptr);
     if (n<=0)
       return gensizeerr(contextptr);
     vecteur v;
@@ -1185,6 +1190,7 @@ namespace giac {
 	  try {
 	    newsol=solve(res-x,*t._IDNTptr,3,contextptr);
 	  } catch(std::runtime_error & err){
+	    last_evaled_argptr(contextptr)=NULL;
 	    newsol.clear();
 	    *logptr(contextptr) << "Unable to solve implicit equation "<< res-x << "=0 in " << t << endl;
 	  }
@@ -1237,6 +1243,7 @@ namespace giac {
 	  try {
 	    newsol=solve(implicitsol,*y._IDNTptr,3,contextptr);
 	  } catch(std::runtime_error & err){
+	    last_evaled_argptr(contextptr)=NULL;
 	    newsol.clear();
 	    *logptr(contextptr) << "Unable to solve implicit equation "<< implicitsol << "=0 in " << y << endl;
 	  }
@@ -1276,7 +1283,8 @@ namespace giac {
 	if (is_undef(tmpsto))
 	  return tmpsto;
 	f=quotesubst(*it,makevecteur(x,y),makevecteur(tplus*x,tplus*y),contextptr);
-	f=normal(f-*it,contextptr);
+	f=recursive_normal(f-*it,contextptr);
+	purgenoassume(tplus,contextptr);
 	if (is_zero(f)){
 	  if (step_info)
 	    gprintf("Order 1 Homogeneous differential equation",vecteur(0),step_info,contextptr);
@@ -1285,7 +1293,7 @@ namespace giac {
 	  tmpsto=sto(doubleassume_and(vecteur(2,0),0,1,false,contextptr),x,contextptr);
 	  if (is_undef(tmpsto))
 	    return tmpsto;
-	  f=normal(quotesubst(*it,y,tplus*x,contextptr)-tplus,contextptr);
+	  f=recursive_normal(quotesubst(*it,y,tplus*x,contextptr)-tplus,contextptr);
 	  purgenoassume(x,contextptr);
 	  // y=tx -> t'x=f
 	  // Singular solutions f(t)=0
