@@ -44,7 +44,7 @@ using namespace std;
 #include <sys/param.h>
 #endif
 
-#if !defined BESTA_OS && !defined NSPIRE // test should always return true
+#if !defined BESTA_OS && !defined NSPIRE && !defined NUMWORKS // test should always return true
 #include <dirent.h>
 #endif
 
@@ -443,6 +443,7 @@ namespace giac {
       readhelp(v,f_name,count,warn);
     return v;
   }
+  // FIXME: aide_cas may end with synonyms (# cmd synonym1 ...)
   void readhelp(vector<aide> & v,const char * f_name,int & count,bool warn){
     count=0;
 #ifndef NSPIRE
@@ -497,8 +498,8 @@ namespace giac {
 	vector<aide>::iterator itpos;
 	for (it=itbeg;it!=itend;++it){
 	  itpos=lower_bound(v.begin(),v.end(),current_aide,alpha_order);
-	  if (itpos!=v.begin()){
-	    --itpos;
+	  if (itpos!=v.end()){
+	    // --itpos;
 	    if (itpos->cmd_name==it->chaine){ // already documented
 	      current_synonymes=itpos->synonymes;
 	      current_blabla=itpos->blabla;
@@ -674,30 +675,31 @@ namespace giac {
   multimap<string,string> html_mtt,html_mall;
   std::vector<std::string> html_vtt,html_vall;
 
+  // WARNING rebuilding caches works with old version of hevea (1.10) but not with hevea 2.29
   // find index nodes in file file
   static bool find_index(const std::string & current_dir,const std::string & file,multimap<std::string,std::string>&mtt,multimap<std::string,std::string>&mall,bool is_index=false,bool warn=false){
     if (access(file.c_str(),R_OK))
       return false;
     ifstream i(file.c_str());
-    // Skip navigation panel
-#if defined VISUALC || defined BESTA_OS
+    // Skip navigation panel 
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
     char * buf=new char[BUFFER_SIZE+1];
 #else
     char buf[BUFFER_SIZE+1];
 #endif
     for (;i && !i.eof();){
       i.getline(buf,BUFFER_SIZE,'\n');
-      string s(buf);
-      if (s=="<!--End of Navigation Panel-->")
+      string s(buf),stmp;
+      if (s=="<!--End of Navigation Panel-->") // latex2html?
 	break;
       int t=int(s.size());
-      if (t>24 && s.substr(t-24,24)=="<LI CLASS=\"li-indexenv\">"){
+      if (t>24 && ((stmp=s.substr(t-24,24))=="<LI CLASS=\"li-indexenv\">" || stmp=="<li class=\"li-indexenv\">")){
 	// hevea file contains index
 	for (;i && !i.eof(); ){
 	  i.getline(buf,BUFFER_SIZE,'\n');
 	  s=buf;
 	  t=int(s.size());
-	  if (t>29 && s.substr(0,29)=="</LI><LI CLASS=\"li-indexenv\">"){
+	  if (t>29 && ((stmp=s.substr(0,29))=="</LI><LI CLASS=\"li-indexenv\">" || stmp=="</li><li class=\"li-indexenv\">")){
 	    s=s.substr(29,s.size()-29);
 	    t=int(s.size());
 	    if (!t || s[0]=='<') // skip index words with special color/font
@@ -709,9 +711,12 @@ namespace giac {
 	      vector<string> hrefs;
 	      for (;;){
 		t=int(s.size());
-		endcmd=int(s.find("<A HREF=\""));
-		if (endcmd<0 || endcmd+9>=t)
-		  break;
+		endcmd=int(s.find("<a href=\""));
+		if (endcmd<0 || endcmd+9>=t){
+		  endcmd=int(s.find("<A HREF=\""));
+		  if (endcmd<0 || endcmd+9>=t)
+		    break;
+		}
 		s=s.substr(endcmd+9,s.size()-endcmd-9);
 		t=int(s.size());
 		endcmd=int(s.find("\""));
@@ -726,7 +731,7 @@ namespace giac {
 		t=int(s.size());
 		if (t<3)
 		  break;
-		if (s.substr(0,3)=="<B>")
+		if (s.substr(0,3)=="<B>" || (t>30 && s.substr(0,29)=="<span style=\"font-weight:bold"))
 		  hrefs.insert(hrefs.begin(),link);
 		else
 		  hrefs.push_back(link);
@@ -739,13 +744,14 @@ namespace giac {
 	      }
 	    } // if (endcmd>2 && endcmd<t)
 	  } // if (t>29 &&...
-	} // end of file
-#if defined VISUALC || defined BESTA_OS
+	} // for (;i && !i.eof();) end of file
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	delete [] buf;
 #endif
 	return true;
-      }
-      if (t>14 && s.substr(t-14,14)=="Index</A></B> "){
+      } // end hevea file with index
+      // latex2html only?
+      if (t>14 && s.substr(t-14,14)=="Index</A></B> "){ 
 	// look in the corresponding index file instead
 	int t1=int(s.find("HREF"))+6;
 	if (t1>=0 && t1<t-16){
@@ -753,13 +759,13 @@ namespace giac {
 	  if (warn)
 	    cerr << "Using index " << s << endl;
 	  find_index(current_dir,current_dir+s,mtt,mall,true,warn);
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	  delete [] buf;
 #endif
 	  return true;
 	}
       }
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
       delete [] buf;
 #endif
     }
@@ -899,7 +905,7 @@ namespace giac {
     return 0;
   }
 
-#if ! (defined VISUALC || defined BESTA_OS || defined NSPIRE || defined NSPIRE_NEWLIB)
+#if ! (defined VISUALC || defined BESTA_OS || defined FREERTOS || defined NSPIRE || defined NSPIRE_NEWLIB)
 #ifdef WIN32
   static int dir_select (const struct dirent *d){
     string s(d->d_name);
@@ -1055,7 +1061,7 @@ namespace giac {
 
   // extern int debug_infolevel;
   static bool get_index_from_cache(const char * filename, multimap<string,string> & multi,bool verbose){
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS  || defined FREERTOS
     char * buf = new char[BUFFER_SIZE];
 #else
     char buf[BUFFER_SIZE];
@@ -1067,7 +1073,7 @@ namespace giac {
       if (!if_mtt || if_mtt.eof()){
 	if (verbose)
 	  cerr << "// Read " << n << " entries from cache " << filename << endl;
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	delete [] buf;
 #endif
 	return true;
@@ -1075,7 +1081,7 @@ namespace giac {
       string first(buf);
       if_mtt.getline(buf,BUFFER_SIZE,char(0xa4));
       if (!if_mtt || if_mtt.eof()){
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	delete [] buf;
 #endif
 	return false;
@@ -1096,15 +1102,13 @@ namespace giac {
 	  multi.clear();
 	  cerr << "Wrong cache! " << filename << endl;
 	  if_mtt.close();
-	  #ifndef RTOS_THREADX
-	  #ifndef BESTA_OS
+#if !defined RTOS_THREADX && !defined BESTA_OS && !defined FREERTOS
 	  if (unlink(filename)==-1)
 	    cerr <<  "You don't have write permissions on " << filename <<".\nYou must ask someone who has write permissions to remove " << filename << endl;
 	  else
 	    cerr << "Cache file "<< filename << " has been deleted" << endl;
-	  #endif
-	  #endif
-#if defined VISUALC || defined BESTA_OS
+#endif
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	  delete [] buf;
 #endif
 	  return false;
@@ -1115,14 +1119,14 @@ namespace giac {
     }
     if (verbose)
       cerr << "// Read " << n << " entries from cache " << filename ;
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
     delete [] buf;
 #endif
     return true;
   }
 
   static bool get_index_from_cache(const char * filename, vector<string> & multi,bool verbose){
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
     char * buf = new char[BUFFER_SIZE];
 #else
     char buf[BUFFER_SIZE];
@@ -1132,21 +1136,18 @@ namespace giac {
     while (if_mtt && !if_mtt.eof()){
       if_mtt.getline(buf,BUFFER_SIZE,char(0xa4));
       if (!if_mtt || if_mtt.eof()){
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 	delete [] buf;
 #endif
 	if (verbose)	
 	  cerr << "// Read " << n << " entries from cache " << filename << endl;
-#if defined VISUALC || defined BESTA_OS
-	delete [] buf;
-#endif
 	return true;
       }
       multi.push_back(buf);
       ++n;
       if_mtt.getline(buf,BUFFER_SIZE,'\n');
     }
-#if defined VISUALC || defined BESTA_OS
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
     delete [] buf;
 #endif
     if (verbose)
