@@ -238,6 +238,12 @@ namespace giac {
     gen xval=x._IDNTptr->eval(1,x,contextptr);
     if (xval!=x){
       _purge(x,contextptr);
+      xval=x._IDNTptr->eval(1,x,contextptr);
+      if (xval!=x){
+	string s="Unable to purge "+x.print(contextptr)+ ", choose another free variable name";
+	*logptr(contextptr) << s << endl;
+	return gensizeerr(s);
+      }
       gen res=residue(g_,x,a,contextptr);
       sto(xval,x,contextptr);
       return res;
@@ -831,6 +837,7 @@ namespace giac {
 	}
 #ifndef NO_STDEXCEPT
       } catch (...){
+	last_evaled_argptr(contextptr)=NULL;
 	return false;
       }
 #endif
@@ -1068,7 +1075,7 @@ namespace giac {
 	  res=0;
 	  return true;
 	}
-	if (intgab_r(g,x,a,b,rational,res,contextptr))
+	if (is_zero(A) && intgab_r(g,x,a,b,rational,res,contextptr))
 	  return true;
 	if (ieo==1){
 	  // simplify g on 0..inf
@@ -1227,6 +1234,8 @@ namespace giac {
     if (!has_evalf(gm,gabd,1,contextptr) || is_zero(gabd))
       gm=simplify(gm,contextptr);
     if (is_constant_wrt(g,x,contextptr)){
+      if (contains(g,x))
+	g=ratnormal(g,contextptr);
       res=g*(b-a);
       return true;
     }
@@ -1374,6 +1383,7 @@ namespace giac {
       }
 #ifndef NO_STDEXCEPT
     } catch (std::runtime_error & ){
+      last_evaled_argptr(contextptr)=NULL;
     }
 #endif
     return false;
@@ -1782,6 +1792,8 @@ namespace giac {
 	    res = P*(-Qa/Ra)+1;
 	    if (!is_zero(res))
 	      res=simplify(pow(res,truen,contextptr)*limit(g,*x._IDNTptr,truea,0,contextptr),contextptr);
+	    if (absint(diffb.val)>100 || absint(diffa.val)>100) 
+	      return false;
 	    if (diffb.val>0){ // b>trueb: add sum(g,x,trueb+1,b-1)
 	      for (int i=0;i<diffb.val;++i)
 		res += simplify(limit(g,*x._IDNTptr,trueb+1+i,0,contextptr),contextptr);
