@@ -11,6 +11,7 @@
 #include "plot.h"
 #include "tex.h"
 #include "mathml.h"
+#include "maple.h"
 #include "giacintl.h"
 
 //#include <fcntl.h>
@@ -66,11 +67,13 @@ namespace giac {
   const double svg_epaisseur1=200; // thickness=(xmax-xmin)/svg_epaisseur1
 
   struct svg_attribut {
+    double ysurx;
     int color;
     int width;
     int epaisseur_point;
     int type_line;
     int type_point;
+    int labelpos;
     bool fill_polygon;
     bool hidden_name;
     bool ie; // Internet explorer flag for SVG
@@ -363,7 +366,10 @@ namespace giac {
     if (itb->type==_FRAC)
       s=gen2mathml(*itb,contextptr); 
     else {
-      if ( (itb->type==_IDNT) || ((itb->type!=_SYMB) && is_positive(*itb,contextptr)) || sommetstr=="<mo>=</mo>")
+      if ( 
+	  !need_parenthesis(*itb) || itb->is_symb_of_sommet(at_of)
+	  // (itb->type==_IDNT) || ((itb->type!=_SYMB) && is_positive(*itb,contextptr)) 
+	  || sommetstr=="<mo>=</mo>")
 	s=gen2mathml(*itb,contextptr);
       else
 	s="<mo>(</mo>"+gen2mathml(*itb,contextptr)+"<mo>)</mo>";
@@ -584,11 +590,98 @@ namespace giac {
     }
   }
 
-  string svg_preamble(double svg_width_cm, double svg_height_cm,bool xml){
-    return svg_preamble(svg_width_cm,svg_height_cm,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,true,xml);
+  void arc_en_ciel(int k,int & r,int & g,int & b){
+    k += 21;
+    k %= 126;
+    if (k<0)
+      k += 126;
+    if (k<21){
+      r=251; g=0; b=12*k;
+    }
+    if (k>=21 && k<42){
+      r=251-(12*(k-21)); g=0; b=251;
+    } 
+    if (k>=42 && k<63){
+      r=0; g=(k-42)*12; b=251;
+    } 
+    if (k>=63 && k<84){
+      r=0; g=251; b=251-(k-63)*12;
+    } 
+    if (k>=84 && k<105){
+      r=(k-84)*12; g=251; b=0;
+    } 
+    if (k>=105 && k<126){
+      r=251; g=251-(k-105)*12; b=0;
+    } 
   }
 
-  string svg_preamble(double svg_width_cm, double svg_height_cm,double xmin,double xmax,double ymin,double ymax,bool ortho,bool xml){
+
+  static string color_string(int color){
+    switch (color) {
+    case 0:
+      return "black";
+    case _RED:
+      return "red";
+    case _GREEN:
+      return "green";
+    case _YELLOW:
+      return "yellow";
+    case _BLUE:
+      return "blue";
+    case _MAGENTA:
+      return "magenta";
+    case _CYAN:
+      return "cyan";
+    case 7:
+      return "white";
+    }
+    int r,g,b;
+    if (color>=0x100 && color<0x17e)
+      color -= 0x100;
+    arc_en_ciel(color,r,g,b);
+    return "rgb("+print_INT_(r)+","+print_INT_(g)+","+print_INT_(b)+")";
+  }
+
+  void pixon2svg(const gen & g,string & s){
+    if (g.type==_VECT){
+      vecteur w=merge_pixon(*g._VECTptr);
+      const_iterateur it=w.begin(),itend=w.end();
+      for (;it!=itend;++it){
+	pixon2svg(*it,s);
+      }
+      return;
+    }
+    if (!g.is_symb_of_sommet(at_pnt))
+      return;
+    gen h=remove_at_pnt(g);
+    if (!h.is_symb_of_sommet(at_pixon))
+      return;
+    h=h._SYMBptr->feuille;
+    if (h.type!=_VECT || h._VECTptr->size()<2)
+      return;
+    vecteur & v=*h._VECTptr;
+    gen x=pixon_size*v[0],y=pixon_size*v[1];
+    int color=0,w=0,he=0;
+    if (v.size()>2)
+      color=v[2].val;
+    if (v.size()>3 && v[3].type==_INT_){
+      if (v[3].val>0) 
+	he=v[3].val; 
+      else 
+	w=-v[3].val;
+    }
+    if (w==0 && he==0){
+      char ch='0'+pixon_size;
+      s += "<rect x=\""+x.print(context0)+"\" y=\""+y.print(context0)+"\" width=\""+ch+"\" height=\""+ch+"\" fill=\""+color_string(color)+"\"/>\n";
+      return;
+    }
+    s += "<rect x=\""+x.print(context0)+"\" y=\""+y.print(context0)+"\" width=\""+print_INT_((w+1)*pixon_size)+"\" height=\""+print_INT_((he+1)*pixon_size)+"\" fill=\""+color_string(color)+"\"/>\n";
+    
+  }
+
+  // before making a user transformation on the frame, 
+  // collect pixon instructions in g
+  string svg_preamble_pixel(const gen &g,double svg_width_cm, double svg_height_cm,double xmin,double xmax,double ymin,double ymax,bool ortho,bool xml){
     double svg_width=xmax-xmin;
     double svg_height=ymax-ymin;
     double x_scale=(xmax-xmin)/10;
@@ -618,13 +711,25 @@ namespace giac {
       // sortie << "<svg ";
     }
     pos=buffer+strlen(buffer);
+    string pixons;
+    pixon2svg(g,pixons);
+    if (!pixons.empty()){
+      sprintf(pos,"width=\"320\" height=\"240\">\n");
+      pos=buffer+strlen(buffer);
+      sprintf(pos,"%s",pixons.c_str());
+      return buffer;
+    }
     if (ortho){
-      sprintf(pos,"preserveAspectRatio=\"xMidyMin meet\" width=\"%.5gcm\" height=\"%.5gcm\" viewBox=\"%.5g %.5g %.5g %.5g\" >\n<g transform=\"translate(0,%.5g) scale(1,-1)\">\n",svg_width_cm+2,svg_height_cm+1,xmin-x_scale,ymin-y_scale,svg_width+3*x_scale,svg_height+2*x_scale,svg_height+2*ymin);
+      sprintf(pos,"width=\"%.5gcm\" height=\"%.5gcm\" ",svg_width_cm+2,svg_height_cm+1);
+      pos=buffer+strlen(buffer);
+      sprintf(pos,"preserveAspectRatio=\"xMidyMin meet\" viewBox=\"%.5g %.5g %.5g %.5g\" >\n<g transform=\"translate(0,%.5g) scale(1,-1)\">\n",xmin-x_scale,ymin-y_scale,svg_width+3*x_scale,svg_height+2*x_scale,svg_height+2*ymin);
       // sortie << "preserveAspectRatio=\"xMidyMin meet\"";
       // sortie << " width=\""<<svg_width_cm+2<<"cm\" height=\""<<svg_height_cm+1<<"cm\" viewBox=\""<<xmin-x_scale<<" "<<ymin-y_scale<<" "<<svg_width+3*x_scale<<" "<<svg_height+2*x_scale<<"\" >\n<g transform=\"translate(0,"<<svg_height+2*ymin<<") scale(1,-1)\">\n";
     }
     else {
-      sprintf(pos,"width=\"%.5gcm\" height=\"%.5gcm\" preserveAspectRatio=\"none\" viewBox=\"%.5g %.5g %.5g %.5g\" >\n<g transform=\"translate(0,%.5g) scale(1,-1)\">\n",svg_width_cm*1.2,svg_height_cm*1.2,xmin-x_scale,ymin-y_scale,svg_width+2*x_scale,svg_height+2*y_scale,svg_height+2*ymin);
+      sprintf(pos,"width=\"%.5gcm\" height=\"%.5gcm\" ",svg_width_cm*1.2,svg_height_cm*1.2);
+      pos=buffer+strlen(buffer);
+      sprintf(pos,"preserveAspectRatio=\"none\" viewBox=\"%.5g %.5g %.5g %.5g\" >\n<g transform=\"translate(0,%.5g) scale(1,-1)\">\n",xmin-x_scale,ymin-y_scale,svg_width+2*x_scale,svg_height+2*y_scale,svg_height+2*ymin);
       // sortie << " width=\""<<svg_width_cm+2<<"cm\" height=\""<<svg_height_cm+1<<"cm\" viewBox=\""<<xmin-x_scale<<" "<<ymin-y_scale<<" "<< svg_width+2*x_scale<<" "<<svg_height+2*y_scale<<'"';
       // sortie << " preserveAspectRatio=\"none\" >" ;
       // sortie << "\n<g transform=\"translate(0,"<<svg_height+2*ymin<<") scale(1,-1)\">\n";
@@ -678,6 +783,18 @@ namespace giac {
     return buffer;
   }
 
+  string svg_preamble_pixel(const gen &g,double svg_width_cm, double svg_height_cm,bool xml){
+    return svg_preamble_pixel(g,svg_width_cm,svg_height_cm,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,true,xml);
+  }
+
+  string svg_preamble(double svg_width_cm, double svg_height_cm,bool xml){
+    return svg_preamble_pixel(0,svg_width_cm, svg_height_cm, xml);
+  }
+
+  string svg_preamble(double svg_width_cm, double svg_height_cm,double xmin,double xmax,double ymin,double ymax,bool ortho,bool xml){
+    return svg_preamble_pixel(0,svg_width_cm,svg_height_cm,xmin,xmax,ymin,ymax,ortho,xml);
+  }
+  
   string svg_grid(){
     return svg_grid(gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax);
   }
@@ -742,25 +859,8 @@ namespace giac {
     return buffer;
   }
 
-
-  static string color_string(svg_attribut attr){
-    switch (attr.color) {
-    case 0:
-      return "black";
-    case _RED:
-      return "red";
-    case _GREEN:
-      return "green";
-    case _YELLOW:
-      return "yellow";
-    case _BLUE:
-      return "blue";
-    case _MAGENTA:
-      return "magenta";
-    case _CYAN:
-      return "cyan";
-    }
-    return "black";
+  static inline string color_string(svg_attribut attr){
+    return color_string(attr.color);
   }
 
 #if 0
@@ -780,6 +880,8 @@ namespace giac {
 #endif
  
   static string svg_text(gen A, string legende, svg_attribut attr,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    if (attr.hidden_name)
+      return "";
     double x_scale=(xmax-xmin)/10;
     double y_scale=(ymax-ymin)/10;
     double fontscale=0.3*x_scale;
@@ -791,19 +893,30 @@ namespace giac {
     if (legende[0]=='\"')
       legende=legende.substr(1,end-1);
     gen x=re(A,contextptr); // should take care of legend position
-    gen y(ymax+ymin-im(A,contextptr));
+    if (x.type!=_DOUBLE_){
+      x=evalf(x,1,contextptr);
+      x=evalf_double(x,1,contextptr);
+    }
     if (is_greater(x,xmax,contextptr))
       x=xmax-x_scale;
     if (is_greater(xmin,x,contextptr))
       x=xmin+x_scale;    
+    gen y(ymax+ymin-im(A,contextptr));
+    if (y.type!=_DOUBLE_){
+      y=evalf(y,1,contextptr);
+      y=evalf_double(y,1,contextptr);
+    }
     if (is_greater(y,ymax,contextptr))
       y=ymax-y_scale;
     if (is_greater(ymin,y,contextptr))
       y=ymin+y_scale;    
     string res= "<text  fill=\"" +color_string(attr)+"\"  x=\""+(x/fontscale).print(contextptr)+"\" y=\""+(y/ratio/fontscale).print(contextptr)
-      +"\" transform=\"translate(0,"+print_DOUBLE_(ymin+ymax,contextptr)+") scale("+print_DOUBLE_(fontscale,contextptr)+","+print_DOUBLE_(-ratio*fontscale,contextptr)+")\" style=\"font-size:1.0pt; text-anchor:end;\">"
-      +legende
-      +"</text>\n";
+      +"\" transform=\"translate(0,"+print_DOUBLE_(ymin+ymax,contextptr)+") scale("+print_DOUBLE_(fontscale,contextptr)+","+print_DOUBLE_(-ratio*fontscale,contextptr)+")\" style=\"font-size:1.4pt; text-anchor:";
+    if (attr.labelpos==0 || attr.labelpos==3)
+      res = res +"start;\">";
+    else
+      res = res +"end;\">";
+    res = res +legende +"</text>\n";
     return res;
   }
   
@@ -819,142 +932,6 @@ namespace giac {
     return res;
   }
   
-  static string svg_circle(const gen & diameter0, const gen & diameter1, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    string s="";
-    gen center=evalf_double((diameter0+diameter1)/2,1,contextptr);
-    gen d01=evalf_double(diameter0-diameter1,1,contextptr);
-    gen di,dr;
-    reim(d01,dr,di,contextptr);
-    double did=di._DOUBLE_val,drd=dr._DOUBLE_val;
-    // COUT << center << " " << d01 << " " << drd << " " << did << endl;
-    double r=std::sqrt(drd*drd+did*did)/2.0;
-    // COUT << r << endl;
-    if (attr.ie){
-      // COUT << xmin << " " << xmax << " " << ymin << " " << ymax << " " << svg_epaisseur1 << " " << attr.width << endl;
-      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
-      // COUT << thickness << endl;
-      s= "<circle stroke-width=\""+print_DOUBLE_(thickness,5)+"\" stroke=\""+color_string(attr);
-    }
-    else
-      s = "<circle vector-effect=\"non-scaling-stroke\" stroke=\""+color_string(attr)+"\"  stroke-width=\""+print_INT_(attr.width);
-    // COUT << s << endl;
-    s = s+"\" fill=\"none\" cx=\""
-      + re(center,contextptr).print(contextptr)+"\" cy=\""
-      + im(center,contextptr).print(contextptr)+"\" r=\""
-      + print_DOUBLE_(r,contextptr)
-      + "\" />\n";
-    if (legende!="")
-      s = s+svg_text(evalf(center+r,1,contextptr),legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    // COUT << s << endl;
-    return s;
-  }
-
-
-  static string svg_segment(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    string s;
-    // gen thickness((xmax+ymax-xmin-ymin)/svg_epaisseur1);
-    A=evalf(A,1,contextptr); B=evalf(B,1,contextptr);
-    if (attr.ie){
-      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
-      s= "<line stroke-width=\""+print_DOUBLE_(thickness,5);
-    }
-    else
-      s= "<line vector-effect=\"non-scaling-stroke\" stroke-width=\""+print_INT_(attr.width);
-    s = s+"\" stroke=\""+color_string(attr)+"\" x1=\""
-      + re(A,contextptr).print(contextptr)+"\" y1=\""
-      + im(A,contextptr).print(contextptr)+"\" x2=\""
-      + re(B,contextptr).print(contextptr)+"\" y2=\""
-      + im(B,contextptr).print(contextptr)+"\"/>\n";
-    s = s+svg_text(B,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    return s;
-  }
-
-  static string svg_point(gen center, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    double svg_width=xmax-xmin;
-    double svg_height=ymax-ymin;
-    gen i=cst_i;
-    gen dx(svg_width/100);
-    gen dy(svg_height/100);
-    switch (attr.type_point << 25){
-    case _POINT_LOSANGE:
-      return svg_segment(center-dx,center-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-i*dy,center+dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+dx,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+i*dy,center-dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_PLUS:
-      return svg_segment(center-dx,center+dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_INVISIBLE:
-      return svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_CARRE:
-      return svg_segment(center-dx-i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center-dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_TRIANGLE:
-      return svg_segment(center-dx-i*dy,center-dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx-i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    case _POINT_POINT:
-      return svg_segment(center,center,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-    }
-    return svg_segment(center-dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
-  }
-
-  static string svg_line(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    gen i=cst_i, C,D;
-    A=evalf(A,1,contextptr); B=evalf(B,1,contextptr);
-    // recherche de l'??quation de la droite
-    if (is_zero(eval(re(A,contextptr)-re(B,contextptr),eval_level(contextptr),contextptr))){
-      gen x=re(A,contextptr);
-      C=x+i*ymin; D=x+i*ymax;
-    } else {
-      gen a=eval((im(A,contextptr)-im(B,contextptr))/(re(A,contextptr)-re(B,contextptr)),eval_level(contextptr),contextptr);
-      gen b=eval(im(A,contextptr)-a*re(A,contextptr),eval_level(contextptr),contextptr);
-      C=xmin+i*(a*xmin+b);
-      D=xmax+i*(a*xmax+b);
-    }
-    return svg_segment(C,D,attr,legende,xmin,xmax,ymin,ymax,contextptr);
-  }
-
-  static string svg_half_line(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    gen i=cst_i, C,D;
-    A=evalf(A,1,contextptr); B=evalf(B,1,contextptr);
-    // recherche de l'??quation de la droite
-    if (is_zero(eval(re(A,contextptr)-re(B,contextptr),eval_level(contextptr),contextptr))){
-      gen x=re(A,contextptr);
-      if (is_positive(eval(im(B,contextptr)-im(A,contextptr),eval_level(contextptr),contextptr),contextptr))
-	C=x+i*ymax;
-      else
-	C=x+i*ymin;
-    } 
-    else {
-      gen a=eval((im(A,contextptr)-im(B,contextptr))/(re(A,contextptr)-re(B,contextptr)),eval_level(contextptr),contextptr);
-      gen b=eval(im(A,contextptr)-a*re(A,contextptr),eval_level(contextptr),contextptr);
-      if (is_positive(eval(im(B,contextptr)-im(A,contextptr),eval_level(contextptr),contextptr),contextptr))
-	C=xmax+i*(a*xmax+b);
-      else
-	C=xmin+i*(a*xmin+b); 
-    }
-    return svg_segment(A,C,attr,legende,xmin,xmax,ymin,ymax,contextptr);
-  }
-
-
-  static string svg_polyline(gen g,svg_attribut attr, string name,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
-    string x="x", y="y", s;
-    int i;
-    g=eval(g,eval_level(contextptr),contextptr);
-    vecteur v=*(g._VECTptr);
-    if (v[0]==v[v.size()-1])
-      s= "<polygon ";
-    else
-      s= "<polyline ";
-    if (attr.ie){
-      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
-      s = s+"stroke-width=\""+print_DOUBLE_(thickness,5);
-    }
-    else
-      s = s+"vector-effect=\"non-scaling-stroke\"  stroke-width=\""+print_INT_(attr.width);
-    s = s+"\" stroke=\""+color_string(attr)+"\" fill=\"none\" points=\"";
-    for (i=0 ; i<signed(v.size())-1 ; i++){
-      s = s+re(evalf(v[i],1,contextptr),contextptr).print(contextptr)+" "+im(evalf(v[i],1,contextptr),contextptr).print(contextptr)+", ";
-    }
-    s = s+re(evalf(v[i],1,contextptr),contextptr).print(contextptr)+" "+im(evalf(v[i],1,contextptr),contextptr).print(contextptr)+"\" /> ";
-    s = s+svg_text(v[i],name,attr,xmin,xmax,ymin,ymax,contextptr);
-    return s;
-  }
-
   // Trace de courbes
   // un point sur 2 sert de point de contr??le
   // on peut sophistiquer
@@ -968,7 +945,10 @@ namespace giac {
     }
     else
       s = s+"vector-effect=\"non-scaling-stroke\"  stroke-width=\""+print_INT_(attr.width);
-    s = s+"\" stroke=\""+color_string(attr)+"\" fill=\"none\" d=\"M";
+    if (attr.fill_polygon)
+      s = s+"\" fill=\""+color_string(attr)+"\" d=\"M";
+    else
+      s = s+"\" stroke=\""+color_string(attr)+"\" fill=\"none\" d=\"M";
     g=evalf(g,1,contextptr);
     vecteur v=*(g._VECTptr);
     for (i=0 ; i<signed(v.size()) ; i++){
@@ -998,8 +978,242 @@ namespace giac {
     return s;
   }
 
+  static string svg_circle(const gen & diameter0, const gen & diameter1, const gen & angle1,const gen & angle2,svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string s="";
+    gen center=evalf_double(_evalc((diameter0+diameter1)/2,contextptr),1,contextptr);
+    gen centerx,centery;
+    reim(center,centerx,centery,contextptr);
+    double cx=evalf_double(centerx,1,contextptr)._DOUBLE_val;
+    double cy=evalf_double(centery,1,contextptr)._DOUBLE_val;
+    gen d01=evalf_double(_evalc(diameter1-diameter0,contextptr),1,contextptr);
+    gen rotg=arg(d01,0);
+    double rot=evalf_double(rotg,1,contextptr)._DOUBLE_val;
+    gen di,dr;
+    reim(d01,dr,di,contextptr);
+    double did=di._DOUBLE_val,drd=dr._DOUBLE_val;
+    // COUT << center << " " << d01 << " " << drd << " " << did << endl;
+    double r=std::sqrt(drd*drd+did*did)/2.0;
+    // COUT << r << endl;
+    double a1=evalf_double(angle1,1,contextptr)._DOUBLE_val+rot;
+    double a2=evalf_double(angle2,1,contextptr)._DOUBLE_val+rot;
+    if (a2<a1) std::swap(a1,a2);
+    bool arc=std::abs(a2-a1-2*M_PI)>1e-4;
+    if (arc){ // bezier curve is safer...
+      int n=100;
+      vecteur v; v.reserve(n+1);
+      complex<double> c(cx,cy);
+      complex<double> ca=std::exp(complex<double>(0,a1));
+      if (attr.fill_polygon){
+	for (double i=0;i<n/2;++i){
+	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
+	}
+      }
+      for (int i=0;i<=n;++i){
+	double theta=a1+i*(a2-a1)/n;
+	v.push_back(gen(c+r*std::exp(complex<double>(0,theta))));
+      }
+      if (attr.fill_polygon){
+	ca=std::exp(complex<double>(0,a2));
+	for (double i=n/2-1;i>=0;--i){
+	  v.push_back(gen(c+(2.0*i*r)/double(n)*ca));
+	}
+      }
+      return svg_bezier_curve(v,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+    }
+    s= string(arc?"<path ":"<circle ");
+    if (attr.ie){
+      // COUT << xmin << " " << xmax << " " << ymin << " " << ymax << " " << svg_epaisseur1 << " " << attr.width << endl;
+      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
+      // COUT << thickness << endl;
+      s = s+" stroke-width=\""+print_DOUBLE_(thickness,5)+"\" stroke=\""+color_string(attr);
+    }
+    else
+      s = s+"vector-effect=\"non-scaling-stroke\" stroke=\""+color_string(attr)+"\"  stroke-width=\""+print_INT_(attr.width);
+    // COUT << s << endl;
+    if (attr.fill_polygon)
+      s = s+"\" fill=\""+color_string(attr)+"\" ";
+    else
+      s = s+"\" fill=\"none\" ";
+    if (arc){
+      // arc of circle: compute start.x, start.y, end.x end.y
+      // from polar coord (radius,a1) and (radius,a2)
+      // largeArcFlag=1 if a2-a1>=M_PI
+      // <path d="M start.x start.y 
+      //          A radius radius rotation largeArcFlag 0 end.x end.y
+      //          L center.x center.y Z">
+      // move to start, arc to end, segment to center, loop to start
+      s = s+" d=\"M "+print_DOUBLE_(cx+r*std::cos(a1),5)+" "+print_DOUBLE_(cy+r*std::sin(a1),5);
+#if 1
+      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" 0"+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
+#else
+      s = s+" A " + print_DOUBLE_(r,5)+ " "+print_DOUBLE_(r,5)+" "+print_DOUBLE_(rot*rad2deg_d,5)+((a2-a1<M_PI)?" 0 1 ":" 1 1 ");
+#endif
+      s = s+ print_DOUBLE_(cx+r*std::cos(a2),5)+" "+print_DOUBLE_(cy+r*std::sin(a2),5);
+      if (attr.fill_polygon)
+	s = s+" L "+print_DOUBLE_(cx,5)+" "+print_DOUBLE_(cy,5)+" Z";
+      s = s+"\" />\n";
+      //CERR << s << endl;
+    }
+    else 
+      s = s + "cx=\""+centerx.print(contextptr)+"\" cy=\""
+	+ centery.print(contextptr)+"\" r=\""
+	+ print_DOUBLE_(r,contextptr)
+	+ "\" />\n";
+    if (legende!="")
+      s = s+svg_text(evalf(center+r,1,contextptr),legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    // COUT << s << endl;
+    return s;
+  }
 
-  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT);
+
+  static string svg_segment(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string s;
+    // gen thickness((xmax+ymax-xmin-ymin)/svg_epaisseur1);
+    A=evalf(A,1,contextptr); B=evalf(B,1,contextptr);
+    if (attr.ie){
+      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
+      s= "<line stroke-width=\""+print_DOUBLE_(thickness,5);
+    }
+    else
+      s= "<line vector-effect=\"non-scaling-stroke\" stroke-width=\""+print_INT_(attr.width);
+    s = s+"\" stroke=\""+color_string(attr)+"\" x1=\""
+      + re(A,contextptr).print(contextptr)+"\" y1=\""
+      + im(A,contextptr).print(contextptr)+"\" x2=\""
+      + re(B,contextptr).print(contextptr)+"\" y2=\""
+      + im(B,contextptr).print(contextptr)+"\"/>\n";
+    //CERR << s << endl;
+    s = s+svg_text(B,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    return s;
+  }
+
+  static string svg_vecteur(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string s;
+    // gen thickness((xmax+ymax-xmin-ymin)/svg_epaisseur1);
+    s = svg_segment(A,B,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+    gen Ax,Ay,Bx,By;
+    reim(A,Ax,Ay,contextptr);
+    reim(B,Bx,By,contextptr); 
+    Ax=evalf_double(Ax,1,contextptr); Bx=evalf_double(Bx,1,contextptr);
+    Ay=evalf_double(Ay,1,contextptr); By=evalf_double(By,1,contextptr);
+    if (Ax.type==_DOUBLE_ && Bx.type==_DOUBLE_ && Ay.type==_DOUBLE_ && By.type==_DOUBLE_){
+      double dx=Ax._DOUBLE_val-Bx._DOUBLE_val,dy=Ay._DOUBLE_val-By._DOUBLE_val;
+      double dxy=std::sqrt(dx*dx+dy/attr.ysurx*dy/attr.ysurx);
+      if (dxy){
+	dxy =dxy/(std::min(5.0,dxy/10.0)+std::max(1.0,double(attr.width)));
+	dx = dx/dxy;
+	dy = dy/dxy;
+	dxy = std::min(xmax-xmin,ymax-ymin)/70.0;
+	dx = dx*dxy;
+	dy = dy*dxy;
+	double dxp=-dy/attr.ysurx,dyp=dx*attr.ysurx; // apparent perpendicular
+	dx = dx*std::sqrt(3.0);
+	dy = dy*std::sqrt(3.0);
+	//CERR << "step3 " << dx << " " << dxp << " " << dx+dxp << " " << dy << endl;
+	double dx1=dx+dxp,dx2=dx-dxp,dy1=dy+dyp,dy2=dy-dyp;
+	gen C=B+gen(dx1,dy1);
+	gen D=B+gen(dx2,dy2);
+	//CERR << C << " " << D << endl;
+	s += svg_segment(B,C,attr,"",xmin,xmax,ymin,ymax,contextptr);
+	s += svg_segment(B,D,attr,"",xmin,xmax,ymin,ymax,contextptr);
+      }
+    }
+    return s;
+  }
+
+  static string svg_point(gen center, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    double svg_width=xmax-xmin;
+    double svg_height=ymax-ymin;
+    gen i=cst_i;
+    gen dx(svg_width/100);
+    gen dy(svg_height/100);
+    switch (attr.type_point << 25){
+    case _POINT_LOSANGE:
+      return svg_segment(center-dx,center-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-i*dy,center+dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+dx,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+i*dy,center-dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_PLUS:
+      return svg_segment(center-dx,center+dx,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_INVISIBLE:
+      return svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_CARRE:
+      return svg_segment(center-dx-i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center+dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center-dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_TRIANGLE:
+      return svg_segment(center-dx-i*dy,center-dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx-i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    case _POINT_POINT: // must make a small segment otherwise nothing visible
+      return svg_segment(center-dx/2,center+dx/2,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+    }
+    return svg_segment(center-dx-i*dy,center+dx+i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_segment(center-dx+i*dy,center+dx-i*dy,attr,"",xmin,xmax,ymin,ymax,contextptr)+svg_text(center,legende,attr,xmin,xmax,ymin,ymax,contextptr);
+  }
+
+  static string svg_line(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    gen i=cst_i, C,D;
+    A=evalf(A,1,contextptr); B=evalf(B,1,contextptr);
+    // recherche de l'??quation de la droite
+    if (is_zero(eval(re(A,contextptr)-re(B,contextptr),eval_level(contextptr),contextptr))){
+      gen x=re(A,contextptr);
+      C=x+i*ymin; D=x+i*ymax;
+    } else {
+      gen a=eval((im(A,contextptr)-im(B,contextptr))/(re(A,contextptr)-re(B,contextptr)),eval_level(contextptr),contextptr);
+      gen b=eval(im(A,contextptr)-a*re(A,contextptr),eval_level(contextptr),contextptr);
+      C=xmin+i*(a*xmin+b);
+      D=xmax+i*(a*xmax+b);
+    }
+    return svg_segment(C,D,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+  }
+
+  static string svg_half_line(gen A, gen B, svg_attribut attr, string legende,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    gen i=cst_i, C,D;
+    A=evalf(eval(A,1,contextptr),1,contextptr); B=evalf(eval(B,1,contextptr),1,contextptr);
+    gen reA,imA,reB,imB;
+    reim(A,reA,imA,contextptr);
+    reim(B,reB,imB,contextptr);
+    // recherche de l'equation de la droite
+    if (is_zero(reA-reB)){
+      gen x=reA;
+      if (is_positive(imB-imA,contextptr))
+	C=x+i*ymax;
+      else
+	C=x+i*ymin;
+    } 
+    else {
+      gen a=(imB-imA)/(reB-reA);
+      gen b=imA-a*reA;
+      if (is_positive(reB-reA,contextptr))
+	C=xmax+i*(a*xmax+b);
+      else
+	C=xmin+i*(a*xmin+b); 
+    }
+    return svg_segment(A,C,attr,legende,xmin,xmax,ymin,ymax,contextptr);
+  }
+
+
+  static string svg_polyline(gen g,svg_attribut attr, string name,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+    string x="x", y="y", s;
+    int i;
+    g=eval(g,eval_level(contextptr),contextptr);
+    vecteur v=*(g._VECTptr);
+    if (v[0]==v[v.size()-1])
+      s= "<polygon ";
+    else
+      s= "<polyline ";
+    if (attr.ie){
+      double thickness=geo_thickness(xmin,xmax,ymin,ymax)/svg_epaisseur1*attr.width;
+      s = s+"stroke-width=\""+print_DOUBLE_(thickness,5);
+    }
+    else
+      s = s+"vector-effect=\"non-scaling-stroke\"  stroke-width=\""+print_INT_(attr.width);
+    if (attr.fill_polygon)
+      s = s+"\" fill=\""+color_string(attr)+"\" points=\"";
+    else
+      s = s+"\" stroke=\""+color_string(attr)+"\" fill=\"none\" points=\"";
+    for (i=0 ; i<signed(v.size())-1 ; i++){
+      s = s+re(evalf(v[i],1,contextptr),contextptr).print(contextptr)+" "+im(evalf(v[i],1,contextptr),contextptr).print(contextptr)+", ";
+    }
+    s = s+re(evalf(v[i],1,contextptr),contextptr).print(contextptr)+" "+im(evalf(v[i],1,contextptr),contextptr).print(contextptr)+"\" /> ";
+    s = s+svg_text(v[i],name,attr,xmin,xmax,ymin,ymax,contextptr);
+    return s;
+  }
+
+
+  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT);
 
   //fonction appelee ssi v est un vecteur
   static string vect2svg(gen v, svg_attribut attr, string name,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
@@ -1010,12 +1224,12 @@ namespace giac {
 	return vect2svg(v[0], attr, name,xmin,xmax,ymin,ymax,contextptr);
       }
       if (v[0].type==_SYMB)
-	return symbolic2svg(*v[0]._SYMBptr,xmin,xmax,ymin,ymax,contextptr);
+	return symbolic2svg(*v[0]._SYMBptr,xmin,xmax,ymin,ymax,attr.ysurx,contextptr);
     }
     if (v.subtype==_GROUP__VECT || v._VECTptr->size()>2)
       return svg_polyline(v, attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_VECTOR__VECT)
-      return svg_segment(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
+      return svg_vecteur(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_LINE__VECT)
       return svg_line(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
     if (v.subtype==_HALFLINE__VECT)
@@ -1027,7 +1241,7 @@ namespace giac {
 #include <emscripten.h>
 #endif
 
-  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){ 
+  static string symbolic2svg(const symbolic & mys,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT){ 
     int color=default_color(contextptr);
     string name="";
     if (mys.sommet==at_pnt){ 
@@ -1067,7 +1281,7 @@ namespace giac {
 	    return 0;
 	});
 #endif
-      svg_attribut attr={color,width+1,epaisseur_point,type_line,type_point,fill_polygon,hidden_name,ie};
+      svg_attribut attr={ysurx,color,width+1,epaisseur_point,type_line,type_point,labelpos,fill_polygon,hidden_name,ie};
       if(v.size()==3)
 	name=v[2].print(contextptr);
       if (v[0].type==_VECT){
@@ -1078,11 +1292,18 @@ namespace giac {
       if (v[0].type==_SYMB){ 
 	symbolic figure=*v[0]._SYMBptr; 
 	if (figure.sommet == at_curve){
-	  gen curve=figure.feuille;
-	  return svg_bezier_curve(curve[1],attr,name,xmin,xmax,ymin,ymax,contextptr);
+	  //CERR << attr.ysurx << endl;
+	  gen curve=figure.feuille[1];
+	  string s;
+	  if (type_line>4 && curve.type==_VECT && curve._VECTptr->size()>2){
+	    vecteur & v = *curve._VECTptr;
+	    s=svg_vecteur(v[v.size()/2],v[v.size()/2+1],attr,"",xmin,xmax,ymin,ymax,contextptr);
+	  }
+	  s=s+svg_bezier_curve(curve,attr,name,xmin,xmax,ymin,ymax,contextptr);
+	  return s;
 	}
 	if (figure.sommet == at_pnt)
-	  return symbolic2svg(figure,xmin,xmax,ymin,ymax,contextptr);
+	  return symbolic2svg(figure,xmin,xmax,ymin,ymax,attr.ysurx,contextptr);
 	if (figure.sommet==at_segment || figure.sommet==at_vector){
 	  gen segment=figure.feuille;
 	  return svg_segment(segment[0],segment[1], attr, name,xmin,xmax,ymin,ymax,contextptr); 
@@ -1097,16 +1318,25 @@ namespace giac {
 	}
 	if (figure.sommet==at_cercle ){
 	  gen diametre;
-	  if (figure.feuille.type==_VECT && !figure.feuille._VECTptr->empty() && figure.feuille._VECTptr->front().type==_VECT)
-	    diametre=figure.feuille._VECTptr->front();
+	  gen angle1=0,angle2=2*M_PI;
+	  gen f=figure.feuille;
+	  if (f.type==_VECT && !f._VECTptr->empty() && f._VECTptr->front().type==_VECT){
+	    if (f._VECTptr->size()>=3){
+	      angle1=f[1]; 
+	      angle2=f[2];
+	    }
+	    diametre=f._VECTptr->front();
+	  }
 	  else
-	    diametre=figure.feuille;
+	    diametre=f;
 	  if (diametre.type==_VECT && diametre._VECTptr->size()>=2){
 	    vecteur v=*diametre._VECTptr;
-	    return svg_circle(v[0],v[1], attr, name,xmin,xmax,ymin,ymax,contextptr);
+	    return svg_circle(v[0],v[1], angle1,angle2,attr, name,xmin,xmax,ymin,ymax,contextptr);
 	  }
 	  return "svg circle error";
 	}
+	if (figure.sommet==at_pixon)
+	  return "";
 	return svg_point(v[0], attr, name,xmin,xmax,ymin,ymax,contextptr); 
       }
       return svg_point(v[0], attr, name,xmin,xmax,ymin,ymax,contextptr); 
@@ -1114,38 +1344,46 @@ namespace giac {
     return "undef";
   }
 
-
-  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT){
+  // preamble added for debugging purpose for svg2doutput in gen.cc
+  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,double ysurx,GIAC_CONTEXT,bool withpreamble){
+    string s;
+    if (withpreamble)
+      s=svg_preamble_pixel(e,10,6,false);
     if (e.type== _SYMB)
-      return symbolic2svg(*e._SYMBptr,xmin,xmax,ymin,ymax,contextptr);
+      return s+symbolic2svg(*e._SYMBptr,xmin,xmax,ymin,ymax,ysurx,contextptr);
     if (e.type==_VECT){
-      string s;
       vecteur v=*e._VECTptr;
       for (int i=0; i<signed(v.size()); i++){
 	if (v[i].type==_SYMB){
 	  symbolic sym=*v[i]._SYMBptr; 
 	  if (sym.sommet==at_pnt)
-	    s=s+symbolic2svg(sym,xmin,xmax,ymin,ymax,contextptr);
+	    s=s+symbolic2svg(sym,xmin,xmax,ymin,ymax,ysurx,contextptr);
 	}
 	if (v[i].type==_VECT){
-	  s=s+gen2svg(v[i],xmin,xmax,ymin,ymax,contextptr);
+	  s=s+gen2svg(v[i],xmin,xmax,ymin,ymax,ysurx,contextptr,false);
 	}
       }
       return s;
     }
     return "error";
   }
-  string gen2svg(const gen &e,GIAC_CONTEXT){
-    return gen2svg(e,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,contextptr);
+  string gen2svg(const gen &e,double xmin,double xmax,double ymin,double ymax,GIAC_CONTEXT,bool withpreamble){
+    return gen2svg(e,xmin,xmax,ymin,ymax,1.0,contextptr,withpreamble);
+  }
+  string gen2svg(const gen &e,GIAC_CONTEXT,bool withpreamble){
+    return gen2svg(e,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,1.0,contextptr,withpreamble);
   }
   gen _svg(const gen & g,GIAC_CONTEXT){
     if ( g.type==_STRNG && g.subtype==-1) return  g;
-    if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>1 && (*g._VECTptr)[1].type==_STRNG){
-      ofstream of((*g._VECTptr)[1]._STRNGptr->c_str());
-      of << gen2svg(g._VECTptr->front(),contextptr) << endl;
-      return plus_one;
+    if (g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>1){
+      if ((*g._VECTptr)[1].type==_STRNG){
+	ofstream of((*g._VECTptr)[1]._STRNGptr->c_str());
+	of << gen2svg(g._VECTptr->front(),contextptr,false) << endl;
+	return plus_one;
+      }
+      return string2gen(gen2svg(g,contextptr,true),false);
     }
-    return string2gen(gen2svg(g,contextptr),false);
+    return string2gen(gen2svg(g,contextptr,false),false);
   }
   static const char _svg_s []="svg";
   static define_unary_function_eval (__svg,&_svg,_svg_s);
@@ -1157,10 +1395,18 @@ namespace giac {
   static string symbolic2mathml(const symbolic & mys, string &svg,GIAC_CONTEXT){
 
     string opstring(mys.sommet.ptr()->print(contextptr));
+    bool of=mys.sommet==at_of;
+    if (of || mys.sommet==at_at) {
+      gen g=mys.feuille[1];
+      if (of)
+	return gen2mathml(mys.feuille[0],svg,contextptr)+"<mo>(</mo>"+gen2mathml(g,svg,contextptr)+"<mo>)</mo>";
+      g+=array_start(contextptr);
+      return "<msub><mrow>"+gen2mathml(mys.feuille[0],svg,contextptr)+"</mrow><mrow>"+gen2mathml(g,svg,contextptr)+"</mrow></msub>";
+    }
     if (opstring!="/" && (mys.sommet.ptr()->texprint || mys.sommet==at_different))  
       return mathml_print(mys,contextptr);
     if (mys.sommet==at_pnt) { 
-      svg=svg+symbolic2svg(mys,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,contextptr);
+      svg=svg+symbolic2svg(mys,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,1.0,contextptr);
       return "<mtext>"+mys.print(contextptr)+"</mtext>";
     }
     gen tmp,value;
