@@ -40,7 +40,7 @@ namespace giac {
 #if defined(GIAC_GENERIC_CONSTANTS) || defined(VISUALC) || defined(x86_64)
   const unary_function_ptr * archive_function_tab(){
     static const unary_function_ptr archive_function_tab_ptr[]={*at_plus,*at_neg,*at_binary_minus,*at_prod,*at_division,*at_inv,*at_pow,*at_exp,*at_ln,*at_abs,*at_arg,*at_pnt,*at_point,*at_segment,*at_sto,*at_sin,
-								*at_cos,*at_tan,*at_asin,*at_acos,*at_atan,*at_sinh,*at_cosh,*at_tanh,*at_asinh,*at_acos,*at_atanh,*at_interval,*at_union,*at_minus,*at_intersect,*at_not,
+								*at_cos,*at_tan,*at_asin,*at_acos,*at_atan,*at_sinh,*at_cosh,*at_tanh,*at_asinh,*at_acosh,*at_atanh,*at_interval,*at_union,*at_minus,*at_intersect,*at_not,
 								*at_and,*at_ou,*at_inferieur_strict,*at_inferieur_egal,*at_superieur_strict,*at_superieur_egal,*at_different,*at_equal,*at_rpn_prog,*at_local,*at_return,*at_Dialog,*at_double_deux_points,*at_pointprod,*at_pointdivision,*at_pointpow,*at_hash,*at_pourcent,*at_tilocal,*at_break,*at_continue,*at_ampersand_times,*at_maple_lib,*at_unit,*at_plot_style,*at_xor,*at_check_type,*at_quote_pow,*at_case,*at_dollar,*at_IFTE,*at_RPN_CASE,*at_RPN_LOCAL,*at_RPN_FOR,*at_RPN_WHILE,*at_NOP,*at_unit,*at_ifte,*at_for,*at_bloc,*at_program,*at_same,*at_increment,*at_decrement,*at_multcrement,*at_divcrement,*at_sq,*at_display,*at_of,*at_at,*at_normalmod,*at_equal2,*at_pointplus,*at_pointminus,0};
     archive_function_tab_length=sizeof(archive_function_tab_ptr)/sizeof(const unary_function_ptr *);
     return archive_function_tab_ptr;
@@ -401,14 +401,18 @@ namespace giac {
     if (arg.type==_IDNT || (arg.type==_SYMB && arg._SYMBptr->sommet!=at_neg && (arg._SYMBptr->sommet!=at_exp || calc_mode(contextptr)!=1) && !arg._SYMBptr->sommet.ptr()->printsommet)){
       argpar=arg.is_symb_of_sommet(at_inv) || (arg.is_symb_of_sommet(at_exp) && abs_calc_mode(contextptr)==38);
       if (argpar) s +='(';
-      if (pui.type==_SYMB || pui.type==_FRAC){
+      if (pui.type==_SYMB || pui.type==_FRAC || pui.type==_CPLX){
 	add_print(s,arg,contextptr);
 	if (argpar) s +=')';
+	if (python_compat(contextptr))
+	  s += "**";
+	else {
 #ifdef GIAC_HAS_STO_38
-	s += '^';
+	  s += '^';
 #else
-	s += __pow.s;
+	  s += __pow.s;
 #endif
+	}
 	s += '(';
 	add_print(s,pui,contextptr);
 	s += ')';
@@ -640,36 +644,30 @@ namespace giac {
     e=gen(v,e.subtype);
   }
 
-  static gen eval_sto(const gen & feuille,std::vector<const char *> & last,int level,GIAC_CONTEXT){ // autoname function
+  static gen eval_sto(const gen & feuille,int level,GIAC_CONTEXT){ // autoname function
     // detect vector/matrix addressing with () parsed as function definition
     // e.g. M(j,k):=j+k+1 parsed as M:=(j,k)->j+k+1
     // these affectations are marked by a subtype==1 by the parser
     // if destination is a matrix
-    if (feuille.type==_VECT && feuille.subtype==_SORTED__VECT && feuille._VECTptr->size()==2 && feuille._VECTptr->front().is_symb_of_sommet(at_program)){
-      gen prog=feuille._VECTptr->front()._SYMBptr->feuille;
+    vecteur & feuillev=*feuille._VECTptr;
+    gen & feuilleback=feuillev.back();
+    if (feuille.type==_VECT && feuille.subtype==_SORTED__VECT && feuillev.size()==2 && feuillev.front().is_symb_of_sommet(at_program)){
+      gen prog=feuillev.front()._SYMBptr->feuille;
       if (prog.type==_VECT && prog._VECTptr->size()==3 && (prog._VECTptr->front()!=_VECT || prog._VECTptr->front()._VECTptr->size()==2)){
 	gen val=prog._VECTptr->back();
-	if (feuille._VECTptr->back().type==_IDNT && feuille._VECTptr->back()._IDNTptr->eval(1,feuille._VECTptr->back(),contextptr).type==_VECT){
-	  prog=symbolic(at_of,makesequence(feuille._VECTptr->back(),prog._VECTptr->front()));
-	  return eval_sto(gen(makevecteur(val,prog),_SORTED__VECT),last,level,contextptr);
+	if (feuilleback.type==_IDNT && feuilleback._IDNTptr->eval(1,feuilleback,contextptr).type==_VECT){
+	  prog=symbolic(at_of,makesequence(feuilleback,prog._VECTptr->front()));
+	  return eval_sto(gen(makevecteur(val,prog),_SORTED__VECT),level,contextptr);
 	}
       }
     }
-    gen ans;
-    gen & feuilleback=feuille._VECTptr->back();
-    vecteur & lastarg=last_evaled_arg(contextptr);
     if ( feuilleback.type==_SYMB && (feuilleback._SYMBptr->sommet==at_unquote || feuilleback._SYMBptr->sommet==at_hash ) ){
-      ans=_sto(feuille.eval(level,contextptr),contextptr);
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      gen ans(_sto(feuille.eval(level,contextptr),contextptr));
       return ans;
     }
-    bool b=show_point(contextptr),quotearg=false;
+    bool & showpoint=show_point(contextptr),b=showpoint,quotearg=false;
     if (b)
-      show_point(false,contextptr);
-    gen e=feuille._VECTptr->front();
+      showpoint=false;
 #ifdef GIAC_HAS_STO_38 // quote STO> E, STO> F, STO>R, STO> X, STO>Y
     if (feuilleback.type==_IDNT){
       const char * ch = feuilleback._IDNTptr->id_name;
@@ -677,20 +675,18 @@ namespace giac {
 	quotearg=true;
     }
 #endif
-    if (!quotearg)
-      e=e.eval(level,contextptr);
+    gen e;
+    if (quotearg)
+      e=feuillev.front();
+    else
+      e=feuillev.front().eval(level,contextptr);
     if (b)
-      show_point(b,contextptr);
+      showpoint=true;
     if (e.type==_SYMB && e._SYMBptr->sommet==at_pnt && e._SYMBptr->feuille.type==_VECT && e._SYMBptr->feuille._VECTptr->size()==2 && (contextptr?!contextptr->previous:!protection_level) )
       eval_sto_pnt_symb(feuille,e,contextptr);
     if ( e.type==_VECT && !e._VECTptr->empty() && e._VECTptr->back().type==_SYMB && e._VECTptr->back()._SYMBptr->sommet==at_pnt && (contextptr?!contextptr->previous:!protection_level))
       eval_sto_pnt_vect(feuilleback,e,contextptr);
-    ans=sto(e,feuilleback,contextptr);
-    if (!last.empty())
-      last.pop_back();
-    if (!lastarg.empty())
-      lastarg.pop_back();
-    return ans;
+    return sto(e,feuilleback,contextptr);
   } // end sommet==at_sto
 
   // http://mitpress.mit.edu/sicp/full-text/book/book-Z-H-34.html#%_sec_5.4
@@ -751,9 +747,10 @@ namespace giac {
   // name changed: BESTA compiler see this as repeated code with same sig as passing the itor itself 
   // - compiler whinges like a wounded pig --
   static void increment_instruction_ptr(const gen * it0,const gen * itend,GIAC_CONTEXT){
+    debug_struct *dbgptr=debug_ptr(contextptr);
     const gen * it=it0;
     for (;it!=itend;++it)
-      increment_instruction(*it,contextptr);
+      increment_instruction(*it,dbgptr);
   }
 
   // nr_eval is slower than usual eval because it saves what is usually saved
@@ -1352,6 +1349,7 @@ namespace giac {
 #ifndef NO_STDEXCEPT
       }
       catch (std::runtime_error & e) {
+	last_evaled_argptr(contextptr)=NULL;
 	res=string2gen(e.what(),false);
 	res.subtype=-1;
 	*destination=res;
@@ -1364,39 +1362,37 @@ namespace giac {
   gen symbolic::eval(int level,const context * contextptr) const {
     if (level==0 || !sommet.ptr())
       return *this;
-    gen ans;
     // FIXME test should be removed later, it's here for tests. See global.cc DEFAULT_EVAL_LEVEL
-    if (eval_level(contextptr)==26)
+    int & elevel=eval_level(contextptr);
+    if (elevel==26)
       return nr_eval(*this,level,contextptr);
-    std::vector<const char *> & last =last_evaled_function_name(contextptr);
-    last.push_back(sommet.ptr()->s);
-    vecteur & lastarg=last_evaled_arg(contextptr);
-    lastarg.push_back(feuille);
     if (sommet==at_sto && feuille.type==_VECT)
-      return eval_sto(feuille,last,level,contextptr);
+      return eval_sto(feuille,level,contextptr);
+    const char * & last =last_evaled_function_name(contextptr);
+    const char * save_last=last;
+    last=sommet.ptr()->s;
+    const gen * & lastarg=last_evaled_argptr(contextptr);
+    const gen * save_lastarg=lastarg;
+    lastarg=&feuille;
+    gen ans;
     if (sommet.quoted()){ 
 #ifndef RTOS_THREADX
       if (feuille.type==_SYMB){ 
 	unary_function_ptr & u=feuille._SYMBptr->sommet;
 	if (u==at_unquote){
 	  ans=sommet(feuille.eval(level,contextptr),contextptr);
-	  if (!last.empty())
-	    last.pop_back();
-	  if (!lastarg.empty())
-	    lastarg.pop_back();
+	  last=save_last;
+	  lastarg=save_lastarg;
 	  return ans;
 	}
 	if (u==at_hash){
 	  ans=sommet(gen(*feuille._SYMBptr->feuille._STRNGptr,contextptr),contextptr);
-	  if (!last.empty())
-	    last.pop_back();
-	  if (!lastarg.empty())
-	    lastarg.pop_back();
+	  last=save_last;
+	  lastarg=save_lastarg;
 	  return ans;
 	}
       }
 #endif
-      int & elevel=eval_level(contextptr);
       int save_level=elevel;
       elevel=level;
 #ifdef NO_STDEXCEPT
@@ -1406,39 +1402,32 @@ namespace giac {
 	ans=sommet(feuille,contextptr);
       }
       catch (std::runtime_error & err){
+	lastarg=&feuille;
 	elevel=save_level;
 	throw(err);
       }
 #endif
       elevel=save_level;
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     } // if (sommet.quoted())
     else {
       // pnt check required because pnt name might be the identifier->recursion
-      if (sommet==at_pnt && feuille.type==_VECT && feuille._VECTptr->size()==3){
+      if (feuille.type==_VECT && sommet==at_pnt && feuille._VECTptr->size()==3){
 	ans=(*sommet.ptr())(gen(vecteur(feuille._VECTptr->begin(),feuille._VECTptr->begin()+2),feuille.subtype).in_eval(level,ans,contextptr)?ans:feuille,contextptr);
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+	last=save_last;
+	lastarg=save_lastarg;
 	return ans;
       }
-      if ((sommet==at_neg) && (feuille.type==_IDNT) && !strcmp(feuille._IDNTptr->id_name,string_infinity)){
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+      if (feuille.type==_IDNT && sommet==at_neg && !strcmp(feuille._IDNTptr->id_name,string_infinity)){
+	last=save_last;
+	lastarg=save_lastarg;
 	return minus_inf;
       }
       if (sommet==at_quote){
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+	last=save_last;
+	lastarg=save_lastarg;
 	return quote(feuille,contextptr);
       }
       ans=(*sommet.ptr())(feuille.in_eval(level,ans,contextptr)?ans:feuille,contextptr);
@@ -1448,10 +1437,8 @@ namespace giac {
 	else
 	ans=(*sommet.ptr())(feuille,contextptr);
       */
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     }
   }
@@ -1518,10 +1505,12 @@ namespace giac {
   gen symbolic::evalf(int level,const context * contextptr) const {
     if (level==0)
       return *this;
-    std::vector<const char *> & last =last_evaled_function_name(contextptr);
-    last.push_back(sommet.ptr()->s);
-    vecteur & lastarg=last_evaled_arg(contextptr);
-    lastarg.push_back(feuille);
+    const char * & last =last_evaled_function_name(contextptr);
+    const char * save_last=last;
+    last=sommet.ptr()->s;
+    const gen * & lastarg=last_evaled_argptr(contextptr);
+    const gen * save_lastarg=lastarg;
+    lastarg=&feuille;
     if (sommet==at_sto){ // autoname function
       gen e=feuille._VECTptr->front().evalf(level,contextptr);
       if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_pnt) && (e._SYMBptr->feuille.type==_VECT) && (e._SYMBptr->feuille._VECTptr->size()==2))
@@ -1535,10 +1524,8 @@ namespace giac {
 	}
 	e=v;
       }
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return sto(e,feuille._VECTptr->back(),contextptr);
     }
     gen ans;
@@ -1555,10 +1542,8 @@ namespace giac {
 	  ans=ans+it->evalf(level,contextptr);
 	}
       }
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     }
     if (sommet==at_prod){
@@ -1571,33 +1556,25 @@ namespace giac {
 	  ans=operator_times(ans,it->evalf(level,contextptr),contextptr);
 	}
       }
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     }
     if (sommet.quoted() && sommet!=at_and && !equalposcomp(plot_sommets,sommet) ){ 
       ans=sommet(feuille,contextptr);
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     }
     else {
       if ((sommet==at_neg) && (feuille.type==_IDNT) && !strcmp(feuille._IDNTptr->id_name,string_infinity)){
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+	last=save_last;
+	lastarg=save_lastarg;
 	return minus_inf;
       }
       if (sommet==at_quote){
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+	last=save_last;
+	lastarg=save_lastarg;
 	return quote(feuille,contextptr);
       }
       if (sommet==at_and || (sommet!=at_cercle && equalposcomp(plot_sommets,sommet))){
@@ -1605,17 +1582,13 @@ namespace giac {
 	// is_inevalf=true;
 	ans=new_ref_symbolic(symbolic(sommet,feuille.evalf(1,contextptr)));
 	// is_inevalf=save_is_inevalf;
-	if (!last.empty())
-	  last.pop_back();
-	if (!lastarg.empty())
-	  lastarg.pop_back();
+	last=save_last;
+	lastarg=save_lastarg;
 	return ans;
       }
       ans=(*sommet.ptr())(feuille.evalf(level,contextptr),contextptr);
-      if (!last.empty())
-	last.pop_back();
-      if (!lastarg.empty())
-	lastarg.pop_back();
+      last=save_last;
+      lastarg=save_lastarg;
       return ans;
     }
   }
